import crispy_forms.helper
import crispy_forms.layout

from django.forms import *


class CommonFormMixin(Form):
    """常用表单基类

    显示一个提交按钮
    """

    submit_button_name = '提交'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = crispy_forms.helper.FormHelper()
        if self.submit_button_name:
            self.helper.add_input(
                crispy_forms.layout.Submit('submit', self.submit_button_name, css_class='btn-primary'))
        self.helper.form_method = 'POST'


class CommonForm(CommonFormMixin, Form):
    pass



# Generated by Django 3.2.8 on 2021-11-28 04:41

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import phy_django.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('phy_django', '0002_configuration'),
    ]

    operations = [
        migrations.AddField(
            model_name='configuration',
            name='user',
            field=phy_django.models.UserField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='用户'),
        ),
    ]



from django.db.transaction import atomic
from rest_framework.decorators import action
from rest_framework.pagination import PageNumberPagination, CursorPagination, \
    LimitOffsetPagination as __LimitOffsetPagination
from rest_framework.viewsets import ModelViewSet as __ModelViewSet

__keep = (
    action, atomic,
    PageNumberPagination, CursorPagination,
)


class LimitOffsetPagination(__LimitOffsetPagination):
    pass


class ModelViewSet(__ModelViewSet):
    """符合我的开发习惯的配置"""
    pagination_class = LimitOffsetPagination



from functools import cached_property

import djmoney.money
import import_export.resources
import import_export.widgets
from import_export.resources import *

from phy_django import models


class IntegerChoiceWidget(import_export.widgets.Widget):
    def __init__(self, *args, choices, **kwargs):
        super().__init__(*args, **kwargs)
        self._choices = choices

    @cached_property
    def _choices_name_to_value(self):
        return {name: value for value, name in self._choices}

    @cached_property
    def _choices_value_to_name(self):
        return {value: name for value, name in self._choices}

    def clean(self, value, row=None, *args, **kwargs):
        return self._choices_name_to_value[value]

    def render(self, value, obj=None):
        return self._choices_value_to_name[value]


class MoneyWidget(import_export.widgets.Widget):
    """用于进行 MoneyField 的导出功能"""

    def render(self, value: djmoney.money.Money, obj=None):
        return super(MoneyWidget, self).render(value.amount, obj)


class ModelResource(import_export.resources.ModelResource):
    """优化后的基类

    对一些字段进行了重定义。
    """

    @classmethod
    def widget_from_django_field(cls, f, default=widgets.Widget):
        result = super().widget_from_django_field(f, default)
        if result != default:
            return result
        for klass_name, widget in cls.WIDGETS_MAP.items():
            klass = getattr(models, klass_name)
            if isinstance(f, klass):
                if isinstance(widget, str):
                    widget = getattr(cls, widget)(f)
                return widget
        return default

    @classmethod
    def field_from_django_field(cls, field_name, django_field, readonly):
        widget = cls.widget_from_django_field(django_field)
        widget_kwargs = cls.widget_kwargs_for_field(field_name)
        if isinstance(django_field, models.IntegerField) and django_field.choices:
            widget = IntegerChoiceWidget
            widget_kwargs.setdefault('choices', django_field.choices)
        if isinstance(django_field, models.MoneyField):
            widget = MoneyWidget
        field = cls.DEFAULT_RESOURCE_FIELD(
            attribute=field_name,
            column_name=django_field.verbose_name,
            widget=widget(**widget_kwargs),
            readonly=readonly,
            default=django_field.default,
        )
        return field



"""
为 ``Django`` 自带的数据类型添加一些符合我的开发习惯的默认值，并添加了一些新的数据类型。
"""
import functools
import typing

import django.contrib
import django.contrib.auth
import django.core.exceptions
import django.db.models
import django.db.models.fields.files
import django.utils.timezone
from django.db.models import *
from django.db.transaction import atomic
from django.utils.functional import classproperty
from djmoney.models.fields import MoneyField

__keep = (atomic,)

if typing.TYPE_CHECKING:
    pass

user_model = django.contrib.auth.get_user_model()


class ForeignKey(ForeignKey):
    """继承自 ``ForeignKey``

    - 不进行数据库的外键约束
    - 禁止删除有外键的条目
    """

    @functools.wraps(ForeignKey.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('on_delete', PROTECT)
        kwargs.setdefault('null', True)
        kwargs.setdefault('blank', True)
        super().__init__(*args, **kwargs)


class FloatField(FloatField):
    """继承自 ``FloatField``

    - 设置默认值为0
    """

    @functools.wraps(FloatField.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('default', 0)
        super().__init__(*args, **kwargs)


class FieldFile(django.db.models.fields.files.FieldFile):
    """
    :meta private:
    """
    pass


class FileField(FileField):
    """继承自 ``FileField``

    - 允许空值
    """
    attr_class = FieldFile

    @functools.wraps(FileField.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('blank', True)
        kwargs.setdefault('null', True)
        super().__init__(*args, **kwargs)


class SmallIntegerField(SmallIntegerField):
    """继承自小整数字段

    - 默认值为 ``0``
    """

    @functools.wraps(SmallIntegerField.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('default', 0)
        super().__init__(*args, **kwargs)


class ShortStringField(CharField):
    """短字符串，继承自 ``CharField``

    - 最长 ``100`` 个字符
    - 默认值为空字符串
    - 允许在 ``admin`` 中传入空值
    """

    @functools.wraps(CharField.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('max_length', 100)
        kwargs.setdefault('default', '')
        kwargs.setdefault('blank', True)
        super().__init__(*args, **kwargs)


class LongStringField(CharField):
    """长字符串，继承自 ``CharField``

    - 最长 ``1000`` 个字符
    - 默认值为空字符串
    - 允许在 ``admin`` 中传入空值
    """

    @functools.wraps(CharField.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('max_length', 1000)
        kwargs.setdefault('default', '')
        super().__init__(*args, **kwargs)


class MoneyField(MoneyField):
    """钱字段，继承自 ``MoneyField``

    - 设置默认值为 ``0``
    - 设置默认单位为人民币
    """

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('default', 0)
        kwargs.setdefault('max_digits', 14)
        kwargs.setdefault('decimal_places', 2)
        kwargs.setdefault('default_currency', 'CNY')
        super().__init__(*args, **kwargs)


class UserField(ForeignKey):
    """用户字段

    - 指向 ``auth.get_user_model()``
    - 允许空值
    - 删除保护

    """

    @functools.wraps(ForeignKey.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('verbose_name', '用户')
        kwargs.setdefault('to', django.contrib.auth.get_user_model())
        kwargs.setdefault('on_delete', PROTECT)
        kwargs.setdefault('null', True)
        kwargs.setdefault('blank', True)
        super().__init__(*args, **kwargs)


class TextField(TextField):
    """文本字段，继承自 ``TextField``

    - 允许空值
    - 设置默认值为空字符串
    """

    @functools.wraps(TextField.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('default', '')
        kwargs.setdefault('blank', True)
        super().__init__(*args, **kwargs)


class BooleanField(BooleanField):
    """布尔字段，继承自 ``BooleanField``

    - 设置默认值为否
    """

    @functools.wraps(BooleanField.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('default', False)
        super().__init__(*args, **kwargs)


DEFAULT_DATETIME = django.utils.timezone.datetime(1970, 1, 1, tzinfo=django.utils.timezone.get_current_timezone())


class DateTimeField(DateTimeField):
    """日期时间字段，继承自 ``DateTimeField``

    - 设置默认时间值为当前时区内的 ``1970.1.1``
    """

    @functools.wraps(DateTimeField.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('default', DEFAULT_DATETIME)
        super().__init__(*args, **kwargs)


class DateField(DateField):
    """日期字段，继承自 ``DateField``

    - 设置默认时间值为当前时区内的 ``1970.1.1``
    """

    @functools.wraps(DateField.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('default', DEFAULT_DATETIME.date())
        super().__init__(*args, **kwargs)


class JSONField(JSONField):
    """Json字段，继承自 ``JsonField``

    - 设置默认值为空字典
    """

    @functools.wraps(JSONField.__init__)
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('default', dict)
        super().__init__(*args, **kwargs)


class DatetimeMixin(Model):
    """
    添加数据入库时间和数据更新时间两个自动字段
    """

    class Meta:
        abstract = True

    created_datetime = django.db.models.DateTimeField(
        verbose_name='数据库入库时间', auto_now_add=True,
    )
    updated_datetime = django.db.models.DateTimeField(
        verbose_name='数据库更新时间', auto_now=True,
    )


class Model(DatetimeMixin, django.db.models.Model):
    """
    优化的模型基类，默认包括入库时间和更新时间字段，并自动将复数名称重定义到单数名称。

    在使用的时候，需要同时继承 ``Model`` 和 ``Model.Meta`` 。
    """

    class BaseMeta:
        abstract = False
        verbose_name = '未定义模型名称'

        # noinspection PyMethodParameters
        @django.utils.functional.classproperty
        def verbose_name_plural(cls):
            return cls.verbose_name

    class Meta(BaseMeta):
        abstract = True


class _MappableIntegerChoiceMeta(django.db.models.enums.ChoicesMeta):
    # noinspection PyMethodParameters
    def __new__(mcs, classname, bases, classdict, **kwds):
        has_meta = 'Meta' in classdict
        if has_meta:
            meta_class = classdict.pop('Meta')
            # noinspection PyProtectedMember
            classdict._member_names.remove('Meta')
        else:
            meta_class = None
        result = super().__new__(mcs, classname, bases, classdict, **kwds)
        if has_meta:
            result.Meta = meta_class
        return result


class MappableIntegerChoice(IntegerChoices, metaclass=_MappableIntegerChoiceMeta):
    """
    可以直接生成 ``Django`` 的 ``IntegerChoiceField`` ，包含选项。

    同时，支持设置默认值，支持定义值到 ``Enum`` 的映射以用于解析。

    Examples:
        .. code-block:: python

            class ExpenseType(MappableIntegerChoice):
                class Meta(MappableIntegerChoice.Meta):
                    default_value = 0
                    mapping = {'Active': 1, 'Inactive': 2}
                default = -1, '未设置'
                other = 0, "其他"
                choices_1 = 1, "选项1"
                choices_2 = 2, "选项2"
    """

    class Meta:
        """
        可以定义默认值和预定义的映射
        """
        default_value = 1
        """默认值

        Examples:
            .. code-block:: python

                class ExpenseType(MappableIntegerChoice):
                    class Meta(MappableIntegerChoice.Meta):
                        default_value = 0
        """

        mapping: dict[str, typing.Any] = {}
        """映射
        
        Examples:
            .. code-block:: python
                
                class ExpenseType(MappableIntegerChoice):
                    class Meta(MappableIntegerChoice.Meta):
                        default_value = 0
                        mapping = {'Active': 1, 'Inactive': 2}
        """

    @classmethod
    def parse(cls, value: str):
        """基于预定义的映射进行数据的转换，用于将本数据库中的值与其他的值进行同步

        映射中的定义值不应当用于界面显示，因为显示在界面中的字符串是可以通过 ``label`` 进行定义的。

        Examples:
            .. code-block:: python

                class MyModel(Model):
                    expense_type = ExpenseType.as_field(verbose_name='费用类型', help_text=__h)
        """
        return cls(cls.Meta.mapping.get(value, cls.Meta.default_value))

    @classmethod
    def as_field(cls, verbose_name: str = '', help_text='', negative=False, default=None):
        """作为一个 ``Django`` 的 ``Model`` 中的一个 ``Field`` 直接插入进去

        Examples:
            .. code-block:: python

                deemed_reseller_category = DeemedResellerCategory.as_field('转销类型')

        """
        klass = SmallIntegerField if negative else PositiveSmallIntegerField
        default = cls.Meta.default_value if default is None else default
        return klass(verbose_name=verbose_name, choices=cls.choices, default=default, help_text=help_text)


class IntegerStringField(PositiveBigIntegerField):
    """
    这个类对 ``String`` 进行了一层封装，可以直接用作 ``Model`` 的属性，在调用时将自动进行属性的转换。

    Examples:
        .. code-block:: python

            class MyModel(Model):
                name = IntegerStringField(default=1, verbose_name=verbose_name)
    """

    def to_python(self, value):
        """
        :meta private:
        """
        if isinstance(value, int):
            return String.int_to_string(value)
        elif isinstance(value, str) or value is None:
            return value
        else:
            raise django.core.exceptions.ValidationError(f'Unknown type: {type(value)} of "{value}')

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def from_db_value(self, value, expression, connection):
        """
        :meta private:
        """
        return self.to_python(value)

    def value_to_string(self, obj):
        """
        :meta private:
        """
        return self.to_python(obj)

    def get_prep_value(self, value):
        """
        :meta private:
        """
        return String.string_to_int(value)

    def get_db_prep_value(self, value: str, connection, prepared=False):
        """
        :meta private:
        """
        return String.string_to_int(value)


class String(Model):
    """
    为字符串建立映射，存入一个总的字符串库。

    对于一些重复性较高的字符串，而又没必要新建一张表来存储，使用此方法可以有效地降低其数据库占用量。
    例如，在对接接口时，对方可能会通过字符串提供5种以上的状态值，而这些状态值我们预先并不能完全定义，就可以采用这种方法进行处理。

    Notes:
        字段将在启动时进行缓存，而每次写入时也会再次检查是否有重复。
        这种方法不适用于大量的字符串，不过对于常规需求（几万个字符串）来说都是可以轻松满足的（MB级内存占用）。
    """

    class Meta:
        indexes = (
            Index(fields=['value']),
        )

    value = CharField(
        max_length=255, unique=True,
    )

    def __str__(self):
        return self.value

    __default: int = 1
    __string_to_int_cache: dict[str, int] = {}
    __int_to_string_cache: dict[int, str] = {}

    @classmethod
    def __init_cache(cls):
        objs = [(o.id, o.value) for o in cls.objects.all()]
        cls.__string_to_int_cache = {value: pk for pk, value in objs}
        cls.__int_to_string_cache = {pk: value for pk, value in objs}

    @classmethod
    def string_to_int(cls, value: typing.Optional[str]) -> int:
        """字符串转整型的过程是一个写入的过程，从缓存中读取编号，或者写入编号"""
        if not cls.__string_to_int_cache:
            cls.__init_cache()
        if value is None:
            return cls.__default
        if value in cls.__string_to_int_cache:
            return cls.__string_to_int_cache[value]
        pk = cls.objects.get_or_create(value=value)[0].id
        cls.__string_to_int_cache[value] = pk
        return pk

    @classmethod
    def int_to_string(cls, value: typing.Optional[int]) -> str:
        """整型转字符串的过程是一个读取的过程，从数据库中查询字符串的编号所对应的值"""
        if not cls.__int_to_string_cache:
            cls.__init_cache()
        return cls.__int_to_string_cache.get(value if value else cls.__default)


class Configuration(Model):
    """键值对的配置，支持逐用户级别的配置

    在使用时不需要直接调用这个类，而是直接使用类方法 ``get`` 和 ``set`` 就可以了。

    用户可以设置为空值。
    """

    class Meta(Model.Meta):
        verbose_name = '配置'
        abstract = False

    user = UserField()
    key = ShortStringField(verbose_name='键')
    value = JSONField(verbose_name='值')

    @staticmethod
    def get(key: str, user=None):
        """从数据库获取一个值。

        用户可以指定为空，则表示获取全局配置项。
        """
        obj, created = Configuration.objects.get_or_create(key=key, user=user)
        return obj.value

    @staticmethod
    def set(key: str, value: dict, user=None):
        """设置一个值到数据库

        用户可以设置为空，则表示设置全局配置项。
        """
        obj, created = Configuration.objects.get_or_create(key=key, user=user)
        obj.value = value
        obj.save()



import django.contrib
import django.contrib.auth
import django.contrib.messages
import django.core.exceptions
import django.http
import django.urls
import django.views.generic
import django.views.generic.detail
import guardian.mixins
from guardian.shortcuts import assign_perm

__keep = (django.urls.reverse_lazy, django.urls.reverse, assign_perm)


class PermissionRequiredMixin(guardian.mixins.PermissionRequiredMixin):
    return_403 = True


class PhyDjangoMixin:
    """一些符合我的开发习惯的配置

    - request的类型提示
    - 可以获取用户对象，同样支持类型提示
    - 将发送信息作为一个方法提供
    """
    request: django.http.HttpRequest

    @property
    def user(self) -> django.contrib.auth.get_user_model():
        return self.request.user

    def message_user(self, message: str, level=django.contrib.messages.INFO):
        django.contrib.messages.add_message(self.request, level, message)


class ContextMixin(PhyDjangoMixin, django.views.generic.base.ContextMixin):
    """
    包含一些全局配置项
    """

    site_name = '我的网站'
    page_title = '首页'
    page_description = ''

    def get_page_description(self):
        if isinstance(self.page_description, str):
            return self.page_description
        if isinstance(self.page_description, (list, tuple)):
            return ''.join([f'<p>{line}</p>' for line in self.page_description])
        raise django.core.exceptions.ImproperlyConfigured(f'page description error: {type(self.page_description)}')

    def get_context_data(self, **kwargs):
        kwargs = super().get_context_data(**kwargs)
        [kwargs.setdefault(k, v) for k, v in {
            'user': self.user,
            'site_name': self.site_name,
            'page_title': self.page_title,
            'page_description': self.get_page_description(),
        }.items()]
        return kwargs


class TemplateView(ContextMixin, django.views.generic.TemplateView):
    pass


class DetailView(ContextMixin, django.views.generic.detail.DetailView):
    pass


class FormView(ContextMixin, django.views.generic.FormView):
    template_name = 'phy_django/form.html'

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        return data


class SingleObjectMixin(django.views.generic.detail.SingleObjectMixin):
    pass


class RedirectView(PhyDjangoMixin, django.views.generic.base.RedirectView):
    pass



import contextlib
from collections import OrderedDict
from pprint import pprint
from typing import Type

import django.contrib.auth.models
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.db import connection
from django.test import TestCase as __TestCase
from django.test.utils import CaptureQueriesContext
from rest_framework.response import Response
from rest_framework.test import APIClient

user_model: Type[django.contrib.auth.models.AbstractUser] = get_user_model()


class TestCase(__TestCase):
    """
    Define a simpler test flow by omit some repeatable works.
    The examples can be find at the app ``blog``.
    """

    @contextlib.contextmanager
    def print_queries(self):
        with CaptureQueriesContext(connection) as ctx:
            yield
            for query in ctx.captured_queries:
                print(query['sql'])

    def setUp(self) -> None:
        self.client = APIClient()
        self.admin_user: user_model = user_model.objects.create_superuser(
            username='test-user', password='test-password')
        self.normal_user: user_model = user_model.objects.create_user(
            username='normal-user', password='test-password')
        self.anonymous_user = AnonymousUser()
        super().setUp()

    def login_with_admin(self):
        self.client.force_authenticate(self.admin_user)

    def login_with_normal(self):
        self.client.force_authenticate(self.normal_user)

    def logout(self):
        self.client.force_authenticate()

    def assert_response(self, response: Response, status_code: int, *test_groups):
        self.assertEqual(response.status_code, status_code)
        try:
            if test_groups:
                if not isinstance(test_groups[0], tuple):
                    test_groups = (test_groups,)
                for args in test_groups:
                    d = response.data
                    for key in args[:-1]:
                        if isinstance(d, (dict, OrderedDict)):
                            self.assertIn(key, d)
                        d = d[key]
                    self.assertEqual(d, args[-1])
            return response
        except AssertionError:
            pprint(response.data)
            raise

    def assert_get(self, path: str, data, status_code: int, *groups):
        return self.assert_response(self.client.get(path, data), status_code, *groups)

    def assert_post(self, path: str, data, status_code: int, *groups):
        return self.assert_response(self.client.post(path, data), status_code, *groups)

    def assert_patch(self, path: str, data, status_code: int, *groups):
        return self.assert_response(self.client.patch(path, data), status_code, *groups)

    def assert_delete(self, path: str, data, status_code: int, *groups):
        return self.assert_response(self.client.delete(path, data), status_code, *groups)


class AdminTestCase(TestCase):
    def setUp(self) -> None:
        super(AdminTestCase, self).setUp()
        self.login_with_admin()



# Generated by Django 3.2.8 on 2021-11-14 07:04

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='String',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_datetime', models.DateTimeField(auto_now_add=True, verbose_name='数据库入库时间')),
                ('updated_datetime', models.DateTimeField(auto_now=True, verbose_name='数据库更新时间')),
                ('value', models.CharField(max_length=255, unique=True)),
            ],
        ),
        migrations.AddIndex(
            model_name='string',
            index=models.Index(fields=['value'], name='phy_django__value_1661eb_idx'),
        ),
    ]



# Generated by Django 3.2.8 on 2021-11-28 04:50

from django.db import migrations
import phy_django.models


class Migration(migrations.Migration):

    dependencies = [
        ('phy_django', '0003_configuration_user'),
    ]

    operations = [
        migrations.AlterField(
            model_name='configuration',
            name='value',
            field=phy_django.models.JSONField(default=dict, verbose_name='值'),
        ),
    ]



import os
import sys
from pathlib import Path


def init_run(application_name='application'):
    """在外部脚本中调用可以开启django运行环境

    Args:
        application_name: 项目名称，也就是装有 ``settings.py`` 的那个包的名称
    """
    sys.path.insert(0, str(Path(__file__).parent.parent))
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', f'{application_name}.settings')
    import django
    django.setup()



import django.contrib.auth
import django.db.models
import django.http
import django_admin_object_button
import django_json_widget.widgets
import guardian.admin
import import_export.admin
from django.contrib.admin import *
from django.contrib.admin import ModelAdmin as _DjangoModelAdmin
from django.core.exceptions import PermissionDenied
from django.shortcuts import redirect
from django.template.response import TemplateResponse
from django.urls import resolve, include, path
from django_admin_object_button import object_button

import phy_django.models
from phy_django.models import Configuration

__keep = (
    object_button, redirect, register, display, action, resolve, PermissionDenied,
    path, include, TemplateResponse,  # 用于添加自定义的View
)


class SuperuserPermissionMixin(_DjangoModelAdmin):
    """添加了一个管理员的权限"""

    @staticmethod
    def has_superuser_permission(request):
        return request.user.is_superuser


class AdminCanModifyObjectLevelPermissionMixin(SuperuserPermissionMixin, guardian.admin.GuardedModelAdmin):
    """仅允许管理员修改对象级别的权限"""

    only_admin_can_manage_object_level_permissions = True
    """是否启用这项功能，如果启用，则仅有管理员才有权限修改对象级别的权限"""

    def has_manage_object_perms_permission(self, request):
        if self.only_admin_can_manage_object_level_permissions and not self.has_superuser_permission(request):
            return False
        return True

    def obj_perms_manage_view(self, request, object_pk):
        if not self.has_manage_object_perms_permission(request):
            raise PermissionDenied
        return super().obj_perms_manage_view(request, object_pk)

    def obj_perms_manage_group_view(self, request, object_pk, group_id):
        if not self.has_manage_object_perms_permission(request):
            raise PermissionDenied
        return super().obj_perms_manage_group_view(request, object_pk, group_id)

    def obj_perms_manage_user_view(self, request, object_pk, user_id):
        if not self.has_manage_object_perms_permission(request):
            raise PermissionDenied
        return super().obj_perms_manage_user_view(request, object_pk, user_id)

    change_form_template = 'admin/change_form.html'


class UseInitialDataMixin(_DjangoModelAdmin):
    """如果在创建新对象时，某个字段为空值，且这个字段设置了初始值，则使用这个初始值。"""

    def save_model(self, request, obj, form, change: bool):
        if not change:
            [getattr(obj, k, None) is None and setattr(obj, k, v)
             for k, v in self.get_changeform_initial_data(request).items()]

        return super().save_model(request, obj, form, change)


class OnlyAdminCanChangeOwnerMixin(guardian.admin.GuardedModelAdmin):
    """仅有管理员可以更改对象的所有权"""
    only_admin_can_change_owner = True
    """如果为真，则仅有所有者有权更改对象所有权字段"""

    def get_readonly_fields(self, request, obj=None):
        result = super().get_readonly_fields(request, obj)

        # 如果仅允许管理员控制所有权，则所有者字段将对非管理员禁用
        if self.only_admin_can_change_owner and self.user_can_access_owned_objects_only \
                and not request.user.is_superuser:
            result = (*result, self.user_owned_objects_field)

        return result


class SetCurrentUserDefaultOwnerMixin(guardian.admin.GuardedModelAdmin):
    """如果启用了guardian的权限管理，则将当前用户设置为表单的初始值"""

    def get_changeform_initial_data(self, request: django.http.HttpRequest):
        result = super().get_changeform_initial_data(request)

        # 如果启用了权限管理，将所有者字段的默认值设置为当前用户
        if self.user_can_access_owned_objects_only:
            result.setdefault(self.user_owned_objects_field, request.user)

        return result


class OnlyAdminCanEditFieldsMixin(_DjangoModelAdmin):
    """仅管理员可以更改的字段"""

    only_admin_editable_fields: tuple[str, ...] = ()
    """仅管理员可以更改的字段"""

    def get_readonly_fields(self, request, obj=None):
        result = super().get_readonly_fields(request, obj)

        # 配置仅管理员才可以更改的字段
        if not request.user.is_superuser:
            result = (*result, *self.only_admin_editable_fields)

        return result


class ModelAdmin(
    OnlyAdminCanEditFieldsMixin,
    SetCurrentUserDefaultOwnerMixin,
    OnlyAdminCanChangeOwnerMixin,
    UseInitialDataMixin,
    AdminCanModifyObjectLevelPermissionMixin,
    import_export.admin.ImportExportMixin,
    SuperuserPermissionMixin,
    django_admin_object_button.ObjectButtonMixin,
    guardian.admin.GuardedModelAdmin,
    _DjangoModelAdmin,
):
    """整合一系列功能的后台管理基类"""

    @staticmethod
    def get_action_name(request: django.http.HttpRequest):
        return resolve(request.path).url_name.rsplit('_', maxsplit=1)[1]

    list_per_page = 50
    user_can_access_owned_objects_only = True

    formfield_overrides = {
        django.db.models.JSONField: {
            'widget': django_json_widget.widgets.JSONEditorWidget,
        },
        phy_django.models.JSONField: {
            'widget': django_json_widget.widgets.JSONEditorWidget,
        },
    }
    """对于不同类型的字段使用不同的对象进行显示"""

    def has_import_permission(self, request: django.http.HttpRequest):
        """导入导出的权限不支持对象级别的权限"""
        codename = django.contrib.auth.get_permission_codename('import', self.opts)
        return request.user.has_perm("%s.%s" % (self.opts.app_label, codename))

    def has_export_permission(self, request):
        """导入导出的权限不支持对象级别的权限"""
        codename = django.contrib.auth.get_permission_codename('export', self.opts)
        return request.user.has_perm("%s.%s" % (self.opts.app_label, codename))

    def has_module_permission(self, request):
        """鉴权以判断是否显示在主界面"""
        # 检查应用级权限：如果用户登录且拥用应用中的任何一个表的任何一种权限，则可见
        if super().has_module_permission(request):
            return True

        # 检查行级权限：如果用户有权访问后台且可以操作其中的一个对象，则可见
        return request.user.is_staff and self.get_queryset(request).exists()


@register(Configuration)
class ConfigurationAdmin(ModelAdmin):
    """键值对配置项的管理后台"""
    list_display = ('id', 'user', 'key', 'value')
    user_can_access_owned_objects_only = True



import setuptools

import phy_django

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="phy-django",
    version=phy_django.version,
    author=phy_django.author,
    author_email=phy_django.author_email,
    description=phy_django.description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=phy_django.url,
    packages=setuptools.find_packages(include=['phy_django*']),
    include_package_data=True,
    install_requires=[
        'django',
        'django-rest-framework',
        'django-q',
        'django-dump-load-utf8',
        'django-admin-object-button',
        'django-silk',
        'django-money',
        'django-registration',
        'django-crispy-forms',
        'django-guardian',
        'django-extensions',
        'django-import-export',
        'django-nested-admin',
        'django-json-widget',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



from rest_framework.serializers import *
from rest_framework.serializers import HyperlinkedModelSerializer as _HyperlinkedModelSerializer
from rest_framework.serializers import ModelSerializer as _ModelSerializer

__keep = (Serializer,)


class _PhyDjangoSerializerMixin(_ModelSerializer):
    class Meta:
        pass


class ModelSerializer(_PhyDjangoSerializerMixin, _ModelSerializer):
    pass


class HyperlinkedModelSerializer(_PhyDjangoSerializerMixin, _HyperlinkedModelSerializer):
    pass



# Generated by Django 3.2.8 on 2021-11-28 04:35

from django.db import migrations, models
import phy_django.models


class Migration(migrations.Migration):

    dependencies = [
        ('phy_django', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Configuration',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_datetime', models.DateTimeField(auto_now_add=True, verbose_name='数据库入库时间')),
                ('updated_datetime', models.DateTimeField(auto_now=True, verbose_name='数据库更新时间')),
                ('key', phy_django.models.ShortStringField(blank=True, default='', max_length=100, verbose_name='键')),
                ('value', models.JSONField(verbose_name='值')),
            ],
            options={
                'verbose_name': '配置',
                'verbose_name_plural': '配置',
                'abstract': False,
            },
        ),
    ]
